"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.equirectangular = equirectangular;
exports.polar2cartesian = polar2cartesian;
exports.dotProduct = dotProduct;
exports.linearSolver = linearSolver;
exports.isInside = isInside;
function equirectangular(center, width) {
  var radius = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 6371000;

  var rad2deg = 180 / Math.PI;
  var dy = width / radius * rad2deg;
  var dx = width / (Math.cos(center[1] / rad2deg) * radius) * rad2deg;
  return {
    forward: function forward(_ref) {
      var _ref2 = _slicedToArray(_ref, 2),
          lng = _ref2[0],
          lat = _ref2[1];

      return [(lng - center[0]) / dx, (lat - center[1]) / dy];
    },
    inverse: function inverse(_ref3) {
      var _ref4 = _slicedToArray(_ref3, 2),
          x = _ref4[0],
          y = _ref4[1];

      return [center[0] + x * dx, center[1] + y * dy];
    }
  };
}

function polar2cartesian(theta) {
  return [Math.sin(theta / 180 * Math.PI), -Math.cos(theta / 180 * Math.PI)];
}

function dotProduct(v1, v2) {
  return v1.reduce(function (sum, e, i) {
    return sum + e * v2[i];
  }, 0);
}

function linearSolver(_ref5, _ref6) {
  var _ref8 = _slicedToArray(_ref5, 2),
      alpha1 = _ref8[0],
      beta1 = _ref8[1];

  var _ref7 = _slicedToArray(_ref6, 2),
      alpha2 = _ref7[0],
      beta2 = _ref7[1];

  var DET = alpha1 * beta2 - alpha2 * beta1;
  return function (d1, d2) {
    return [(beta2 * d1 - beta1 * d2) / DET, (-alpha2 * d1 + alpha1 * d2) / DET];
  };
}

function isInside(_ref9, linearRing) {
  var _ref10 = _slicedToArray(_ref9, 2),
      lng = _ref10[0],
      lat = _ref10[1];

  var isInside = false;
  for (var i = 1; i < linearRing.length; i++) {
    var deltaYplus = linearRing[i][1] - lat;
    var deltaYminus = lat - linearRing[i - 1][1];
    if (deltaYplus > 0 && deltaYminus <= 0) continue;
    if (deltaYplus <= 0 && deltaYminus > 0) continue;
    var deltaX = (deltaYplus * linearRing[i - 1][0] + deltaYminus * linearRing[i][0]) / (deltaYplus + deltaYminus) - lng;
    if (deltaX <= 0) continue;
    isInside = !isInside;
  }
  return isInside;
}